using System.ComponentModel;
using System.IO.Ports;

namespace SerialPortExampleCS;

public partial class Form1 : Form
{

    SerialPort _port;

    public BindingList<string> PortNames { get; }

    public BindingList<int> Baudrates { get; }

    public Form1()
    {
        InitializeComponent();
        _port = new SerialPort();
        _port.ErrorReceived += Port_ErrorReceived;
        _port.DataReceived += Port_DataReceived;
        PortNames = new();
        RefreshPortNames();
        cbbComPorts.DataSource = PortNames;
        Baudrates = new();
        RefreshBaudrates();
        cbbBaudrates.DataSource = Baudrates;

        cbbComPorts.SelectionChangeCommitted += CbbComPorts_SelectionChangeCommitted;
        cbbBaudrates.SelectionChangeCommitted += CbbBaudrates_SelectionChangeCommitted;
    }

    private void RefreshPortNames()
    {
        PortNames.Clear();
        foreach (var portName in SerialPort.GetPortNames())
        {
            if (portName != "COM1")
                PortNames.Add(portName);
        }
    }

    private void RefreshBaudrates()
    {
        Baudrates.Add(9600);
        Baudrates.Add(19200);
        Baudrates.Add(38400);
        Baudrates.Add(115200);
    }

    private void CbbComPorts_SelectionChangeCommitted(object? sender, EventArgs e)
    {
        if (_port.IsOpen)
            _port.Close();
        _port.PortName = Convert.ToString(cbbComPorts.SelectedItem) ?? "";
        _port.Open();
    }

    private void CbbBaudrates_SelectionChangeCommitted(object? sender, EventArgs e)
    {
        //if (_port.IsOpen)
        //    _port.Close();
        _port.BaudRate = Convert.ToInt32(cbbBaudrates.SelectedItem);
        // _port.Open();
    }

    private void btnSendCmd_Click(object sender, EventArgs e)
    {
        SendCmd();
    }

    private void tbCmd_KeyDown(object sender, KeyEventArgs e)
    {
        if (e.KeyCode == Keys.Enter)
        {
            e.Handled = true;
            e.SuppressKeyPress = true;
            SendCmd();
        }
    }

    private void SendCmd()
    {
        if (!_port.IsOpen)
        {
            _port.PortName = Convert.ToString(cbbComPorts.SelectedItem) ?? "";
            _port.BaudRate = Convert.ToInt32(cbbBaudrates.SelectedItem);
            _port.Open();
        }
        _port.WriteLine(tbCmd.Text);
        tbCmd.Text = "";
    }

    private void Port_DataReceived(object sender, SerialDataReceivedEventArgs e)
    {
        var data = _port.ReadExisting();
        BeginInvoke(() => TextBox1.AppendText(data));
    }

    private void Port_ErrorReceived(object sender, SerialErrorReceivedEventArgs e)
    {
        BeginInvoke(() => TextBox1.AppendText("error received"));
    }

}
